@@ -4,8 +4,8 @@ import time
 import cx_Oracle
 import os
 import os.path
-from constants import *              # !!! Change this to 'import src.umbitlib.constants' for testing and back to 'import constants' for production upload
-from dates import TRGTMON_YYYYMMDD   # !!! Change this to 'import src.umbitlib.dates' for testing and back to 'import dates' for production upload
+from constants import *  # !!! Change this to 'import src.umbitlib.constants' for testing and back to 'import constants' for production upload
+from dates import TRGTMON_YYYYMMDD  # !!! Change this to 'import src.umbitlib.dates' for testing and back to 'import dates' for production upload
 import warnings
 
 
@@ -13,29 +13,61 @@ import warnings
 ## ORACLE CONNECTION FUNCTIONS
 ####################################################################################
 
+def excel_credentials():   
+    """
+    Summary:
+        Function to retrieve UofU credentials saved in specified Excel doc in G drive.
+
+    Returns:
+        username and password from Excel doc in tuple.
+
+    Example:        
+        username, password = excel_credentials()
+    """    
+    try:  
+        data_source_file_oracle = "\\\\hsc-evs03.ad.utah.edu\\users\\" + \
+            os.getlogin() + "\\Data Sources\\UIDPWD.xlsx"
+        wb_credentials_oracle = load_workbook(data_source_file_oracle)
+        sheet_oracle = wb_credentials_oracle['Sheet1']
+        username = sheet_oracle['A2'].value
+        password = sheet_oracle['B2'].value
+        return username, password
+    except:
+        raise TypeError('Excel workbook with credentials not found.')
+    
 def oracle_connection(username: str=None, password: str=None)->cx_Oracle.Connection:
     """
     Summary:
-        Function to initialize connection to Oracle.\n
-        Uses stored Excel credentials by default.\n
-        But accommodates credentials passed as variables.
+        Function to initialize connection to Oracle. Uses stored Excel credentials by default.\n
+        If no Excel credentials are found, attempts to access environment variables stored as 'ORACLE_USERNAME' and 'ORACLE_PASSWORD'. \n
+        User may override default checks by manually entering username and password via key entry or passed environment variables.
 
     Returns:
-        cx_Oracle.Connection
+        cx_Oracle.Connection: As "connection_oracle"
 
     Note:
         Setting a variable equal to function grants all cx_Oracle functions (i.e. .cursor())
 
     Example:        
-        Set connection variable: conn = oracle_connection() \n
+        Set connection variable: conn = init_oracle_conn() 
         Set cursor variable: cursor = conn.cursor()
         Save query results to dataframe: df = pd.read_sql_query(sqlQueryString, conn)
     """    
 
     if not username and not password:
-        username = ORACLE_USER
-        password = ORACLE_PASS
-        
+        try:
+            username, password = excel_credentials()
+        except:
+            try:
+                username = os.environ['ORACLE_USERNAME']
+                password = os.environ['ORACLE_PASSWORD']     
+            except:
+                try:
+                    username = ORACLE_USER
+                    password = ORACLE_PASS
+                except:
+                    raise TypeError('Excel credentials workbook and environment variables failed. Please try manually entering credentials.')  
+
     if password and username:
         try: 
             oracle_connection = cx_Oracle.connect(
@@ -43,7 +75,7 @@ def oracle_connection(username: str=None, password: str=None)->cx_Oracle.Connect
             print('Connection successful.')
             return oracle_connection  
         except:
-            raise TypeError('Connection failed. Credentials disallowed.')
+            raise TypeError('Connection failed. Supplied credentials disallowed.')
     else:
         raise TypeError('Must supply both username AND password.')
 
